@@ -1,14 +1,14 @@
 """Utility functions."""
 import uuid
 import functools
-from typing import Dict, Callable, Union, Type, Tuple, List, Any, Optional
+from typing import Optional, Union, Type, Iterable, MutableSequence, List, Tuple, Mapping, \
+    Callable, Any, Reversible, ByteString
 from warnings import warn
 
 from gemd.entity.base_entity import BaseEntity
 from gemd.entity.dict_serializable import DictSerializable
 from gemd.entity.link_by_uid import LinkByUID
 
-from collections.abc import Reversible, Iterable, ByteString
 from toolz import concatv
 
 
@@ -38,7 +38,7 @@ def set_uuids(obj, scope):
     return
 
 
-def _cached_isinstance(
+def cached_isinstance(
         obj: object,
         class_or_tuple: Union[Type, Tuple[Union[Type, Tuple[Type]]]]) -> bool:
     """
@@ -61,6 +61,10 @@ def _cached_isinstance(
     return _cached_issubclass(obj_type, class_or_tuple)
 
 
+# Older unreleased versions of citrine-python referenced the private method directly
+_cached_isinstance = cached_isinstance
+
+
 @functools.lru_cache(maxsize=None)
 def _cached_issubclass(
         cls: Type,
@@ -70,8 +74,8 @@ def _cached_issubclass(
 
     Parameters
     ----------
-    obj: object
-
+    cls: type
+        The class to evaluate
     class_or_tuple: Union[Type, Tuple[Union[Type, Tuple[Type]]]]
         A single type, a tuple of types (potentially nested)
 
@@ -87,7 +91,7 @@ def _cached_issubclass(
 def _substitute(thing: Any,
                 sub: Callable[[object], object],
                 applies: Callable[[object], bool],
-                visited: Dict[object, object] = None) -> object:
+                visited: Mapping[object, object] = None) -> object:
     """
     Generic recursive substitute function.
 
@@ -99,7 +103,7 @@ def _substitute(thing: Any,
     thing: Any
         The object to traverse with substitution.
     sub: Callable[[object], object]
-        Function which provides substitute for value; should not have side-effects.
+        Function which provides substitute for value; should not have side effects.
     applies: Callable[[object], bool]
         Function which defines the domain for the sub function to be invoked.
 
@@ -116,14 +120,14 @@ def _substitute(thing: Any,
     else:
         replacement = thing
 
-    if _cached_isinstance(replacement, list):
+    if cached_isinstance(replacement, MutableSequence):
         new = [_substitute(x, sub, applies, visited) for x in replacement]
-    elif _cached_isinstance(replacement, tuple):
+    elif cached_isinstance(replacement, Tuple):
         new = tuple(_substitute(x, sub, applies, visited) for x in replacement)
-    elif _cached_isinstance(replacement, dict):
+    elif cached_isinstance(replacement, Mapping):
         new = {_substitute(k, sub, applies, visited): _substitute(v, sub, applies, visited)
                for k, v in replacement.items()}
-    elif _cached_isinstance(replacement, DictSerializable):
+    elif cached_isinstance(replacement, DictSerializable):
         new_attrs = {_substitute(k, sub, applies, visited): _substitute(v, sub, applies, visited)
                      for k, v in replacement.as_dict().items()}
         new = replacement.build(new_attrs)
@@ -139,7 +143,7 @@ def _substitute(thing: Any,
 def _substitute_inplace(thing: Any,
                         sub: Callable[[object], object],
                         applies: Callable[[object], bool],
-                        visited: Dict[object, object] = None) -> object:
+                        visited: Mapping[object, object] = None) -> object:
     """
     Generic recursive in-place substitute function.
 
@@ -151,17 +155,17 @@ def _substitute_inplace(thing: Any,
     thing: Any
         The object to traverse with substitution.
     sub: Callable[[object], object]
-        Function which provides substitute for value; should not have side-effects.
+        Function which provides substitute for value; should not have side effects.
     applies: Callable[[object], bool]
         Function which defines the domain for the sub function to be invoked.
 
     """
     def _key(obj):
-        if _cached_isinstance(obj, (float, int, str)):
+        if cached_isinstance(obj, (float, int, str)):
             return None
         elif obj.__hash__ is not None:
             return obj
-        elif _cached_isinstance(obj, Iterable) and not _cached_isinstance(obj, ByteString):
+        elif cached_isinstance(obj, Iterable) and not cached_isinstance(obj, ByteString):
             return id(obj)
         else:
             return None  # pragma: no cover  Fallback for caution's sake
@@ -177,13 +181,13 @@ def _substitute_inplace(thing: Any,
     if orig_key is not None:
         visited[orig_key] = thing  # Store before we start recursing
 
-    if _cached_isinstance(thing, list):  # Change list in place
+    if cached_isinstance(thing, MutableSequence):  # Change list in place
         for i, x in enumerate(thing):
             thing[i] = _substitute_inplace(x, sub, applies, visited)
-    elif _cached_isinstance(thing, tuple):  # Tuples are immutable; regenerate
+    elif cached_isinstance(thing, Tuple):  # Tuples are immutable; regenerate
         thing = tuple(_substitute_inplace(x, sub, applies, visited) for x in thing)
         visited[orig_key] = thing  # We mutated it
-    elif _cached_isinstance(thing, dict):  # Change dict in place, both keys & values
+    elif cached_isinstance(thing, Mapping):  # Change dict in place, both keys & values
         remove = set()  # Store todos because can't mutate a dict in a loop
         update = dict()
         for k, v in thing.items():
@@ -197,7 +201,7 @@ def _substitute_inplace(thing: Any,
         for k in remove:
             thing.pop(k, None)
         thing.update(update)
-    elif _cached_isinstance(thing, DictSerializable):
+    elif cached_isinstance(thing, DictSerializable):
         for k, v in thing.as_dict().items():  # Assume key can't change b/c it's an attribute
             new_v = _substitute_inplace(v, sub, applies, visited)
             if id(v) != id(new_v):
@@ -242,7 +246,7 @@ def _setter_by_attribute(clazz: type, attribute: str) -> Callable:
     return setter
 
 
-def make_index(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]):
+def make_index(obj: Union[Iterable, BaseEntity, DictSerializable]):
     """
     Generates an index that can be used for the substitute_objects method.
 
@@ -252,8 +256,8 @@ def make_index(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]):
 
     Parameters
     ----------
-    obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]
-        target container (dict, list, ..) from which to create an index of GEMD objects
+    obj: Union[Iterable, Mapping, BaseEntity, DictSerializable]
+        target container (dict, list, ...) from which to create an index of GEMD objects
 
     """
     def _make_index(_obj: BaseEntity):
@@ -307,7 +311,7 @@ def substitute_links(obj: Any,
 
     return method(obj,
                   sub=lambda o: o.to_link(scope=scope, allow_fallback=allow_fallback),
-                  applies=lambda o: o is not obj and _cached_isinstance(o, BaseEntity))
+                  applies=lambda o: o is not obj and cached_isinstance(o, BaseEntity))
 
 
 def substitute_objects(obj,
@@ -337,10 +341,10 @@ def substitute_objects(obj,
 
     return method(obj,
                   sub=lambda l: index.get(l, l),
-                  applies=lambda o: _cached_isinstance(o, LinkByUID))
+                  applies=lambda o: cached_isinstance(o, LinkByUID))
 
 
-def flatten(obj, scope=None):
+def flatten(obj, scope=None) -> List[BaseEntity]:
     """
     Flatten a BaseEntity (or array of them) into a list of objects connected by LinkByUID objects.
 
@@ -383,7 +387,7 @@ def flatten(obj, scope=None):
         if len(uids) == 0:
             raise ValueError(f"No UID for {base_obj}; pass `flatten` a `scope` to set one")
 
-        # if none of the uids are known, then its a new object and we should return it
+        # if none of the uids are known, then it's a new object and we should return it
         if not any(uid in known_uids for uid in uids):
             to_return = [base_obj]
 
@@ -397,7 +401,7 @@ def flatten(obj, scope=None):
     return sorted([substitute_links(x) for x in res], key=lambda x: writable_sort_order(x))
 
 
-def recursive_foreach(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable],
+def recursive_foreach(obj: Union[Iterable, BaseEntity, DictSerializable],
                       func: Callable[[BaseEntity], None],
                       *,
                       apply_first=False):
@@ -410,7 +414,7 @@ def recursive_foreach(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable
 
     Parameters
     ----------
-    obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]
+    obj: Union[Iterable, Mapping, BaseEntity, DictSerializable]
         target of the operation
     func: Callable[[BaseEntity], None]
         to apply to each contained BaseEntity
@@ -433,38 +437,38 @@ def recursive_foreach(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable
             else:
                 seen.add(this)
 
-        if apply_first and _cached_isinstance(this, BaseEntity):
+        if apply_first and cached_isinstance(this, BaseEntity):
             func(this)
 
-        if _cached_isinstance(this, dict):
+        if cached_isinstance(this, Mapping):
             for x in concatv(this.keys(), this.values()):
                 queue.append(x)
-        elif _cached_isinstance(this, DictSerializable):
+        elif cached_isinstance(this, DictSerializable):
             for k, x in this.__dict__.items():
                 queue.append(x)
-        elif _cached_isinstance(this, Iterable) \
-                and not _cached_isinstance(this, (str, ByteString)):
+        elif cached_isinstance(this, Iterable) \
+                and not cached_isinstance(this, (str, ByteString)):
             for x in this:
                 queue.append(x)
 
-        if not apply_first and _cached_isinstance(this, BaseEntity):
+        if not apply_first and cached_isinstance(this, BaseEntity):
             func(this)
 
     return
 
 
-def recursive_flatmap(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable],
-                      func: Callable[[BaseEntity], Union[List, Tuple]],
+def recursive_flatmap(obj: Union[Iterable, BaseEntity, DictSerializable],
+                      func: Callable[[BaseEntity], Iterable],
                       *,
-                      unidirectional=True):
+                      unidirectional=True) -> List:
     """
     Recursively apply and accumulate a list-valued function to BaseEntity members.
 
     Parameters
     ----------
-    obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]
+    obj: Union[Iterable, Mapping, BaseEntity, DictSerializable]
         target of the operation
-    func: Callable[[BaseEntity], Union[List[Any], Tuple[Any]]]
+    func: Callable[[BaseEntity], Iterable]
         function to apply; must be list-valued
     unidirectional: bool
         only recurse through the writeable direction of bidirectional links
@@ -488,20 +492,20 @@ def recursive_flatmap(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable
             else:
                 seen.add(this)
 
-        if _cached_isinstance(this, BaseEntity):
+        if cached_isinstance(this, BaseEntity):
             res.extend(func(this))
 
-        if _cached_isinstance(this, dict):
+        if cached_isinstance(this, Mapping):
             queue.extend(concatv(this.keys(), this.values()))
-        elif _cached_isinstance(this, DictSerializable):
+        elif cached_isinstance(this, DictSerializable):
             for k, x in sorted(this.__dict__.items()):
-                if unidirectional and _cached_isinstance(this, BaseEntity) and k in this.skip:
+                if unidirectional and cached_isinstance(this, BaseEntity) and k in this.skip:
                     continue
                 queue.append(x)
-        elif _cached_isinstance(this, Reversible):
+        elif cached_isinstance(this, Reversible):
             queue.extend(reversed(this))  # Preserve order of the list/tuple
-        elif _cached_isinstance(this, Iterable) \
-                and not _cached_isinstance(this, (str, ByteString)):
+        elif cached_isinstance(this, Iterable) \
+                and not cached_isinstance(this, (str, ByteString)):
             queue.extend(this)  # No control over order
 
     return res
@@ -514,9 +518,9 @@ def writable_sort_order(key: Union[BaseEntity, str]) -> int:
     from gemd.entity.template import ConditionTemplate, MaterialTemplate, MeasurementTemplate, \
         ParameterTemplate, ProcessTemplate, PropertyTemplate
 
-    if _cached_isinstance(key, BaseEntity):
+    if cached_isinstance(key, BaseEntity):
         typ = key.typ
-    elif _cached_isinstance(key, str):
+    elif cached_isinstance(key, str):
         typ = key
     else:
         raise ValueError("Can ony sort BaseEntities and type strings, not {}".format(key))
